@@ -2,14 +2,15 @@
    Copyright (C) 1997-9 Eddie Kohler, eddietwo@lcs.mit.edu
    This file is part of the GIF library.
 
-   The GIF library is free software*. It is distributed under the GNU Public
-   License, version 2 or later; you can copy, distribute, or alter it at will,
-   as long as this notice is kept intact and this source code is made
+   The GIF library is free software*. It is distributed under the GNU General
+   Public License, version 2 or later; you can copy, distribute, or alter it
+   at will, as long as this notice is kept intact and this source code is made
    available. There is no warranty, express or implied.
 
-   *The LZW compression method used by GIFs is patented. Unisys, the patent
-   holder, allows the compression algorithm to be used without a license in
-   software distributed at no cost to the user. */
+   *There is a patent on the LZW compression method used by GIFs, and included
+   in gifwrite.c. Unisys, the patent holder, allows the compression algorithm
+   to be used without a license in software distributed at no cost to the
+   user. The decompression algorithm is not patented. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -114,7 +115,7 @@ Gif_NewComment(void)
 
 
 Gif_Extension *
-Gif_NewExtension(int kind, char *app_name)
+Gif_NewExtension(int kind, const char *app_name)
 {
   Gif_Extension *gfex = Gif_New(Gif_Extension);
   if (!gfex) return 0;
@@ -134,7 +135,7 @@ Gif_NewExtension(int kind, char *app_name)
 
 
 char *
-Gif_CopyString(char *s)
+Gif_CopyString(const char *s)
 {
   int l;
   char *copy;
@@ -195,7 +196,7 @@ Gif_AddCommentTake(Gif_Comment *gfcom, char *x, int xlen)
 
 
 int
-Gif_AddComment(Gif_Comment *gfcom, char *x, int xlen)
+Gif_AddComment(Gif_Comment *gfcom, const char *x, int xlen)
 {
   char *new_x;
   if (xlen < 0) xlen = strlen(x);
@@ -246,11 +247,12 @@ Gif_CalculateScreenSize(Gif_Stream *gfs, int force)
   
   for (i = 0; i < gfs->nimages; i++) {
     Gif_Image *gfi = gfs->images[i];
-    if (gfi->left != 0 || gfi->top != 0) continue;
-    if (screen_width < gfi->width)
-      screen_width = gfi->width;
-    if (screen_height < gfi->height)
-      screen_height = gfi->height;
+    /* 17.Dec.1999 - I find this old behavior annoying. */
+    /* if (gfi->left != 0 || gfi->top != 0) continue; */
+    if (screen_width < gfi->left + gfi->width)
+      screen_width = gfi->left + gfi->width;
+    if (screen_height < gfi->top + gfi->height)
+      screen_height = gfi->top + gfi->height;
   }
   
   /* Only use the default 640x480 screen size if we are being forced to create
@@ -447,9 +449,9 @@ Gif_DeleteImage(Gif_Image *gfi)
   Gif_DeleteArray(gfi->img);
   if (gfi->compressed && gfi->free_compressed)
     (*gfi->free_compressed)((void *)gfi->compressed);
-  Gif_Delete(gfi);
   if (gfi->user_data && gfi->free_user_data)
     (*gfi->free_user_data)(gfi->user_data);
+  Gif_Delete(gfi);
 }
 
 
@@ -736,7 +738,7 @@ Gif_Debug(char *x, ...)
   va_list val;
   va_start(val, x);
   vfprintf(stderr, x, val);
-  fputc('\n', stderr);
+  fputc(' ', stderr);
   va_end(val);
 }
 
