@@ -6,6 +6,9 @@
 #include <stdarg.h>
 #include <assert.h>
 #include <X11/Xatom.h>
+#ifdef HAVE_XINERAMA
+#include <X11/extensions/Xinerama.h>
+#endif
 
 static Options onormal;
 Options *ocurrent;
@@ -31,7 +34,7 @@ Gif_Stream *bars_slideshow;
 static const char *bars_slideshow_text = 0;
 
 int nports;
-Port *ports;
+Port **ports;
 
 fd_set x_socket_set;
 int max_x_socket;
@@ -123,6 +126,7 @@ Appearance:\n\
   +noiconify          Don't let anyone iconify the warning window.\n\
   ready-picture=GIF-FILE, okp=GIF-FILE   Show GIF animation on the `OK' window.\n\
   rest-picture=GIF-FILE, rp=GIF-FILE     Show GIF animation on resting window.\n\
+  title=TITLE         Set xwrits window title to TITLE.\n\
   +top                Keep the warning windows on top of the window stack.\n\
   warning-picture=GIF-FILE, wp=GIF-FILE  Show GIF animation on the warning\n\
                       window.\n\
@@ -169,14 +173,51 @@ message(const char *format, ...)
 
 /* ports and peers */
 
-Port *
-find_port(Display *display)
+static Port *
+add_port(const char *display_name, Display *display, int screen_number,
+	 Port *master)
 {
-  int i;
-  for (i = 0; i < nports; i++)
-    if (ports[i].display == display)
-      return &ports[i];
-  return 0;
+    Port *p = (Port *)xmalloc(sizeof(Port));
+    ports = (Port **)xrealloc(ports, sizeof(Port *) * (nports + 1));
+    if (!ports || !p)
+	error("out of memory!");
+    ports[nports] = p;
+    memset(p, 0, sizeof(Port));
+    p->display_name = display_name;
+    p->display = display;
+    p->screen_number = screen_number;
+    p->master = (master ? master : p);
+    p->nslaves = 0;
+    if (master)
+	master->nslaves++;
+    p->port_number = nports;
+    nports++;
+    return ports[nports - 1];
+}
+
+Port *
+find_port(Display *display, Window window)
+{
+    int screen_number, i;
+
+    /* first, look using only 'display' */
+    for (i = 0; i < nports; i++)
+	if (ports[i]->display == display && ports[i]->display_unique)
+	    return ports[i]->master;
+
+    /* if display not unique (or not found), try 'screen_number' also */
+    screen_number = 0;
+    if (window) {
+	XWindowAttributes attr;
+	if (XGetWindowAttributes(display, window, &attr) != 0)
+	    screen_number = XScreenNumberOfScreen(attr.screen);
+    }
+    for (i = 0; i < nports; i++)
+	if (ports[i]->display == display && ports[i]->screen_number == screen_number)
+	    return ports[i]->master;
+
+    /* nothing found */
+    return 0;
 }
 
 void
@@ -208,19 +249,19 @@ notify_peers_rest(void)
 {
   int i, j;
   for (i = 0; i < nports; i++)
-    for (j = 0; j < ports[i].npeers; j++)
-      if (check_xwrits_window(&ports[i], ports[i].peers[j])) {
+    for (j = 0; j < ports[i]->npeers; j++)
+      if (check_xwrits_window(ports[i], ports[i]->peers[j])) {
 	XEvent e;
 	e.xclient.type = ClientMessage;
-	e.xclient.window = ports[i].peers[j];
-	e.xclient.message_type = ports[i].xwrits_break_atom;
+	e.xclient.window = ports[i]->peers[j];
+	e.xclient.message_type = ports[i]->xwrits_break_atom;
 	e.xclient.format = 32;
 	e.xclient.data.l[0] = 1;
-	XSendEvent(ports[i].display, ports[i].peers[j], True, 0, &e);
+	XSendEvent(ports[i]->display, ports[i]->peers[j], True, 0, &e);
       } else {
 	/* remove peer */
-	ports[i].peers[j] = ports[i].peers[ ports[i].npeers - 1 ];
-	ports[i].npeers--;
+	ports[i]->peers[j] = ports[i]->peers[ ports[i]->npeers - 1 ];
+	ports[i]->npeers--;
 	j--;
       }
 }
@@ -281,11 +322,12 @@ default_x_processing(XEvent *e)
 {
   Hand *h;
   Display *display = e->xany.display;
-  Port *port = find_port(display);
+  Port *port;
   
   switch (e->type) {
     
    case ConfigureNotify:
+    port = find_port(display, e->xconfigure.window);
     h = window_to_hand(port, e->xconfigure.window, 1);
     if (!h) break;
     h->configured = 1;
@@ -297,11 +339,13 @@ default_x_processing(XEvent *e)
     break;
     
    case ReparentNotify:
+    port = find_port(display, e->xreparent.window);
     if ((h = window_to_hand(port, e->xreparent.window, 1)))
       find_root_child(h);
     break;
     
    case MapNotify:
+    port = find_port(display, e->xmap.window);
     if ((h = window_to_hand(port, e->xmap.window, 1))) {
       draw_slide(h);
       h->mapped = 1;
@@ -309,11 +353,13 @@ default_x_processing(XEvent *e)
     break;
     
    case UnmapNotify:
-    if ((h = window_to_hand(port, e->xmap.window, 1)))
+    port = find_port(display, e->xunmap.window);
+    if ((h = window_to_hand(port, e->xunmap.window, 1)))
       h->mapped = 0;
     break;
     
    case VisibilityNotify:
+    port = find_port(display, e->xvisibility.window);
     if ((h = window_to_hand(port, e->xvisibility.window, 0))) {
       if (e->xvisibility.state == VisibilityUnobscured)
 	h->obscured = 0;
@@ -323,6 +369,7 @@ default_x_processing(XEvent *e)
     break;
     
    case Expose:
+    port = find_port(display, e->xexpose.window);
     h = window_to_hand(port, e->xexpose.window, 0);
     if (e->xexpose.count == 0 && h && h->clock)
       draw_clock(h, 0);
@@ -330,6 +377,7 @@ default_x_processing(XEvent *e)
     
    case ClientMessage:
     /* change e->type depending on the message */
+    port = find_port(display, e->xclient.window);
     h = window_to_hand(port, e->xclient.window, 0);
     if (h) {
       if (e->xclient.message_type == port->wm_protocols_atom
@@ -350,6 +398,7 @@ default_x_processing(XEvent *e)
    case CreateNotify: {
      struct timeval now;
      xwGETTIME(now);
+     port = find_port(display, e->xcreatewindow.window);
      watch_keystrokes(port, e->xcreatewindow.window, &now);
      break;
    }
@@ -358,6 +407,7 @@ default_x_processing(XEvent *e)
     /* We must unschedule any IdleSelect events for fear of selecting input on
        a destroyed window. There is a race condition here because X
        communication is asynchronous. */
+    port = find_port(display, e->xdestroywindow.window);
     unschedule_data(A_IDLE_SELECT, (void *)port,
 		    (void *)e->xdestroywindow.window);
     if (verbose)
@@ -532,9 +582,9 @@ slideshow_text_append_built_in(Options *o, char *built_in)
 	  (lis ? ";" : ""), built_in);
   
   if (!o->prev || o->prev->slideshow_text != o->slideshow_text)
-    xfree(o->slideshow_text);
+    xfree((char *)o->slideshow_text);
   if (!o->prev || o->prev->icon_slideshow_text != o->icon_slideshow_text)
-    xfree(o->icon_slideshow_text);
+    xfree((char *)o->icon_slideshow_text);
   
   o->slideshow_text = s;
   o->icon_slideshow_text = is;
@@ -593,13 +643,10 @@ parse_options(int pargc, char **pargv)
       o->clock = optparse_yesno;
     
     else if (optparse(s, "display", 1, "ss", &arg)) {
-      if (nports == 1 && ports[0].display_name == 0)
-	ports[0].display_name = arg;
-      else {
-	nports++;
-	ports = (Port *)xrealloc(ports, sizeof(Port) * nports);
-	ports[nports-1].display_name = arg;
-      }
+      if (nports == 1 && ports[0]->display_name == 0)
+	ports[0]->display_name = arg;
+      else
+	add_port(arg, 0, 0, 0);
     
     } else if (optparse(s, "finger", 1, "tS", &arg)
 	      || optparse(s, "flipoff", 1, "tS", &arg)) {
@@ -666,7 +713,9 @@ parse_options(int pargc, char **pargv)
       ;
     else if (optparse(s, "ready-picture", 3, "ss", &ready_slideshow_text))
       ;
-    
+
+    else if (optparse(s, "title", 2, "ss", &o->window_title))
+      ;
     else if (optparse(s, "typetime", 1, "st", &normal_type_time))
       ;
     else if (optparse(s, "top", 2, "t"))
@@ -807,6 +856,9 @@ default_settings(void)
   onormal.slideshow_text = onormal.icon_slideshow_text = 0;
   slideshow_text_append_built_in(&onormal, "clench");
   slideshow_text_append_built_in(&onormal, "spread");
+
+  /* window title */
+  onormal.window_title = "xwrits";
   
   /* clock tick time functions */
   /* 20 seconds seems like a reasonable clock tick time, even though it'll
@@ -829,23 +881,68 @@ default_settings(void)
 #endif
 
 static void
-initialize_port(int portno, Display *display, int screen_number)
+initialize_slave_port(Port *port)
+{
+    Port *m = port->master;
+    assert(m->port_number < port->port_number);
+    port->x_socket = m->x_socket;
+    port->root_window = m->root_window;
+    port->display_unique = 0;
+    port->visual = m->visual;
+    port->depth = m->depth;
+    port->colormap = m->colormap;
+    port->black = m->black;
+    port->white = m->white;
+    port->font = m->font;
+    port->gfx = m->gfx;
+    port->wm_delete_window_atom = m->wm_delete_window_atom;
+    port->wm_protocols_atom = m->wm_protocols_atom;
+    port->mwm_hints_atom = m->mwm_hints_atom;
+    port->xwrits_window_atom = m->xwrits_window_atom;
+    port->xwrits_notify_peer_atom = m->xwrits_notify_peer_atom;
+    port->xwrits_break_atom = m->xwrits_break_atom;
+    port->hands = port->icon_hands = 0;
+    port->permanent_hand = m->permanent_hand;
+    port->drawable = m->drawable;
+    port->clock_fore_gc = m->clock_fore_gc;
+    port->clock_hand_gc = m->clock_hand_gc;
+    port->white_gc = m->white_gc;
+    port->peers = 0;
+    port->npeers = 0;
+    port->peers_capacity = 0;
+    port->icon_width = port->icon_height = 0;
+    port->last_mouse_root = None;
+    port->bars_pixmap = None;
+}
+
+static void
+initialize_port(int portno)
 {
   XVisualInfo visi_template;
   int nv, i;
   XVisualInfo *v;
   XVisualInfo *best_v = 0;
   VisualID default_visualid;
-  Port *port = &ports[portno];
+  Port *port = ports[portno];
+  Display *display = port->display;
+  int screen_number = port->screen_number;
+
+  /* check that relevant fields have been initialized */
+  assert(display && screen_number >= 0 && screen_number < ScreenCount(display) && port->port_number == portno);
+  
+  /* initialize slaves specially */
+  if (port->master != port) {
+      initialize_slave_port(port);
+      return;
+  }
   
   /* initialize Port fields */
-  port->display = display;
   port->x_socket = ConnectionNumber(display);
-  port->screen_number = screen_number;
   port->root_window = RootWindow(display, screen_number);
-  port->width = DisplayWidth(display, screen_number);
-  port->height = DisplayHeight(display, screen_number);
-  port->port_number = portno;
+  port->display_unique = 1;
+  for (i = 0; i < nports && port->display_unique; i++)
+      if (i != portno && ports[i]->display == display && ports[i]->master == ports[i])
+	  port->display_unique = 0;
 
   /* set X socket */
   FD_SET(port->x_socket, &x_socket_set);
@@ -884,8 +981,9 @@ initialize_port(int portno, Display *display, int screen_number)
       port->colormap = DefaultColormap(display, screen_number);
     
   }
-  
-  if (v) XFree(v);
+
+  if (v)
+      XFree(v);
   
   /* set up black_pixel and white_pixel */
   {
@@ -905,7 +1003,7 @@ initialize_port(int portno, Display *display, int screen_number)
   
   /* set gfx */
   port->gfx = Gif_NewXContextFromVisual
-    (display, port->screen_number, port->visual, port->depth, port->colormap);
+    (display, screen_number, port->visual, port->depth, port->colormap);
 
   /* set atoms */
   port->wm_delete_window_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);
@@ -1052,13 +1150,56 @@ main(int argc, char *argv[])
   init_scheduler();
   
   srand((getpid() + 1) * time(0));
-  
-  nports = 1;
-  ports = (Port *)xmalloc(sizeof(Port));
+
+  add_port(0, 0, 0, 0);
   
   /* parse options. remove first argument = program name */
   default_settings();  
   parse_options(argc - 1, argv + 1);
+
+  /* At this point, all ports have 'display_name' valid and everything else
+     invalid. Open displays, check multiscreen */
+  orig_nports = nports;
+  for (i = 0; i < orig_nports; i++) {
+      Display *display = XOpenDisplay(ports[i]->display_name);
+      if (!display)
+	  error("can't open display `%s'", ports[i]->display_name);
+      ports[i]->display = display;
+      if (!multiscreen)
+	  ports[i]->screen_number = DefaultScreen(display);
+      else if (ScreenCount(display) > 0) {
+	  ports[i]->screen_number = 0;
+	  for (j = 1; j < ScreenCount(display); j++)
+	      add_port(ports[i]->display_name, ports[i]->display, j, 0);
+      }
+  }
+
+  /* Now check screen dimensions and Xinerama, if available. */
+  orig_nports = nports;
+  for (i = 0; i < orig_nports; i++) {
+      Port *p = ports[i];
+#ifdef HAVE_XINERAMA
+      int j, nxsi;
+      XineramaScreenInfo *xsi = 0;
+      if (XineramaQueryExtension(p->display, &j, &nxsi))
+	  xsi = XineramaQueryScreens(p->display, &nxsi);
+      if (xsi) {
+	  for (j = 0; j < nxsi; j++) {
+	      Port *q = (j == 0 ? p : add_port(p->display_name, p->display, p->screen_number, p));
+	      q->left = xsi[j].x_org;
+	      q->top = xsi[j].y_org;
+	      q->width = xsi[j].width;
+	      q->height = xsi[j].height;
+	  }
+	  XFree(xsi);
+      } else {
+#endif
+      p->width = DisplayWidth(p->display, p->screen_number);
+      p->height = DisplayHeight(p->display, p->screen_number);
+#ifdef HAVE_XINERAMA
+      }
+#endif
+  }
   
   /* check global options */
   if (strlen(lock_password) >= MAX_PASSWORD_SIZE)
@@ -1099,27 +1240,11 @@ main(int argc, char *argv[])
   ready_icon_slideshow = parse_slideshow(ready_icon_slideshow_text, 1, force_mono);
   ocurrent = &onormal;
 
-  /* set up displays */
+  /* create ports */
   FD_ZERO(&x_socket_set);
   max_x_socket = 0;
-  orig_nports = nports;
-  for (i = 0; i < orig_nports; i++) {
-    Display *display = XOpenDisplay(ports[i].display_name);
-    if (!display)
-	error("can't open display `%s'", ports[i].display_name);
-    if (!multiscreen)
-	initialize_port(i, display, DefaultScreen(display));
-    else
-	for (j = 0; j < ScreenCount(display); j++)
-	    if (j == 0)
-		initialize_port(i, display, j);
-	    else {
-		nports++;
-		ports = (Port *)xrealloc(ports, sizeof(Port) * nports);
-		ports[nports-1].display_name = ports[i].display_name;
-		initialize_port(nports-1, display, j);
-	    }
-  }
+  for (i = 0; i < nports; i++)
+    initialize_port(i);
 
   /* initialize pictures using first hand */
   if (lock_possible) {
@@ -1130,7 +1255,7 @@ main(int argc, char *argv[])
     if (bars_slideshow_text) {
       bars_slideshow = parse_slideshow(bars_slideshow_text, 1, force_mono);
       for (i = 0; i < nports; i++)
-        ports[i].bars_pixmap = Gif_XImage(ports[i].gfx, bars_slideshow, 0);
+        ports[i]->bars_pixmap = Gif_XImage(ports[i]->gfx, bars_slideshow, 0);
     }
   }
   
@@ -1138,7 +1263,8 @@ main(int argc, char *argv[])
   xwGETTIME(now);
   XSetErrorHandler(x_error_handler);
   for (i = 0; i < nports; i++)
-    watch_keystrokes(&ports[i], ports[i].root_window, &now);
+      if (ports[i]->master == ports[i])
+	  watch_keystrokes(ports[i], ports[i]->root_window, &now);
   
   /* start mouse checking */
   if (check_mouse) {
